#define _SVID_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/msg.h>
#include <signal.h>
#include "common.h"

enum Constants
{
    TIMEOUT = 30
};

static int isTimeout = 0;

void alarmHandler(int s)
{
    signal(SIGALRM, alarmHandler);
    printf("\nTimeout!\n");
    isTimeout = 1;
}

void exitHandler(int s)
{
    signal(s, exitHandler);
    printf("\nExit request!\n");
    isTimeout = 1;
}

void printMessage(Message * msg)
{
    printf("\n");
    printf("Name     : %s\n", msg->name);
    printf("Last name: %s\n", msg->lastName);
    printf("Age      : %d\n", msg->age);
    printf("Email    : %s\n", msg->email);
    printf("\n");
    printf("\n");
}

int main()
{
    const key_t key = getKey();
    const int queueId = msgget(key, 0666|IPC_CREAT|IPC_EXCL);
    if (queueId == -1)
    {
        fprintf(stderr, "Message queue get failed.\n");
        exit(EXIT_FAILURE);
    }

    signal(SIGALRM, alarmHandler);
    signal(SIGINT, exitHandler);
    signal(SIGHUP, exitHandler);

    size_t size = 0;
    SerializedMessage * serializedMessage = NULL;

    while (!isTimeout)
    {
        alarm(TIMEOUT);
        errno = 0;
        ssize_t recSize = msgrcv(queueId, serializedMessage, size, 0, 0);
        alarm(0);
        if (recSize == -1)
        {
            switch(errno)
            {
                case E2BIG:
                    size += 20;
                    serializedMessage = (SerializedMessage*)
                        realloc(serializedMessage, sizeof(SerializedMessage) + size);
                    break;
                case EINTR:
                    printf("Cought a signal.\n");
                    break;
                default:
                    fprintf(stderr, "Failed to receive message.\n");
                    msgctl(queueId, IPC_RMID, NULL);
                    exit(EXIT_FAILURE);
            }
        }
        else
        {
            Message * msg = deserializeMessage(serializedMessage);
            printMessage(msg);
            freeMessage(msg);
            free(msg);
        }
    }

    printf("Terminating.\n");
	msgctl(queueId, IPC_RMID, NULL);
    free(serializedMessage);

    return EXIT_SUCCESS;
}


